from PyQt5.QtGui import QIcon, QPixmap


def create_icon(icon_path: str = ":/pyqt/source/images/New.png"):
    icon = QIcon()
    icon.addPixmap(QPixmap(icon_path), QIcon.Normal, QIcon.Off)
    return icon


def colorTup2Str(self, value: tuple) -> str:
    if value is None:
        return None
    strcolor = '#'
    for i in value:
        strcolor += hex(int(i))[-2:].replace('x', '0')
    return strcolor



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'mplwidgets.ui'
#
# Created by: PyQt5 UI code generator 5.15.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MatplotlibMainWindow(object):
    def setupUi(self, MatplotlibMainWindow):
        MatplotlibMainWindow.setObjectName("MatplotlibMainWindow")
        MatplotlibMainWindow.resize(800, 600)
        self.central_widget = QtWidgets.QWidget(MatplotlibMainWindow)
        self.central_widget.setObjectName("central_widget")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.central_widget)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.matplotlib_widget = PMMatplotlibQt5Widget(self.central_widget)
        self.matplotlib_widget.setObjectName("matplotlib_widget")
        self.horizontalLayout.addWidget(self.matplotlib_widget)
        MatplotlibMainWindow.setCentralWidget(self.central_widget)
        self.statusbar = QtWidgets.QStatusBar(MatplotlibMainWindow)
        self.statusbar.setObjectName("statusbar")
        MatplotlibMainWindow.setStatusBar(self.statusbar)
        self.toolBar = QtWidgets.QToolBar(MatplotlibMainWindow)
        self.toolBar.setMovable(False)
        self.toolBar.setFloatable(False)
        self.toolBar.setObjectName("toolBar")
        MatplotlibMainWindow.addToolBar(QtCore.Qt.TopToolBarArea, self.toolBar)
        self.dockWidget = QtWidgets.QDockWidget(MatplotlibMainWindow)
        self.dockWidget.setFeatures(QtWidgets.QDockWidget.DockWidgetFloatable|QtWidgets.QDockWidget.DockWidgetMovable)
        self.dockWidget.setObjectName("dockWidget")
        self.dockWidgetContents_2 = QtWidgets.QWidget()
        self.dockWidgetContents_2.setObjectName("dockWidgetContents_2")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.dockWidgetContents_2)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.mpl_settings_panel = SettingsPanel(self.dockWidgetContents_2)
        self.mpl_settings_panel.setObjectName("mpl_settings_panel")
        self.verticalLayout_2.addWidget(self.mpl_settings_panel)
        self.dockWidget.setWidget(self.dockWidgetContents_2)
        MatplotlibMainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(2), self.dockWidget)
        self.dockWidget.raise_()

        self.retranslateUi(MatplotlibMainWindow)
        QtCore.QMetaObject.connectSlotsByName(MatplotlibMainWindow)

    def retranslateUi(self, MatplotlibMainWindow):
        _translate = QtCore.QCoreApplication.translate
        MatplotlibMainWindow.setWindowTitle(_translate("MatplotlibMainWindow", "MainWindow"))
        self.toolBar.setWindowTitle(_translate("MatplotlibMainWindow", "toolBar"))
from pmgwidgets import PMMatplotlibQt5Widget, SettingsPanel



from typing import List, Dict, Tuple

from PyQt5.QtGui import QCloseEvent
from PyQt5.QtWidgets import QWidget, QPushButton, QLineEdit, QLabel, QVBoxLayout, QHBoxLayout, QApplication, \
    QColorDialog, QRadioButton, QCheckBox, QComboBox, QSpacerItem, QSizePolicy
import warnings
import time
import string
import sys


class BaseParamWidget(QWidget):
    '''
    基础参数控件的类型。所有的参数控件都在其上派生而来。
    '''

    def __init__(self, layout_dir='v'):
        super(BaseParamWidget, self).__init__()
        if layout_dir == 'v':
            self.central_layout = QVBoxLayout()
        else:
            self.central_layout = QHBoxLayout()
        self.setLayout(self.central_layout)
        self.central_layout.setContentsMargins(0, 0, 0, 0)

        self.on_para_change = None
        self.__app = None  # SciApp。初始化控件的时候指定，并且在调用set_app的时候传入。

    def para_changed(self):
        if (self.on_para_change is not None) and (self.__app is not None):
            self.on_para_change(self.__app)

    def set_app(self, app):
        self.__app = app

    def is_key(self, event, type=''):
        '''
        'dir':判断方向键
        'alpha':判断是否为26个字母
        'hex':判断是否为十六进制数字或者字母
        'digit':判断是否为数字0~9
        'valid':包含数字、字母或者退格键。
        '''

        type = type.lower()
        if type == '':
            return True
        elif type.startswith('dir'):
            return event.keysym.lower() in ('left', 'right', 'up', 'down')
        elif type.startswith('alpha'):
            return event.keysym in string.ascii_lowercase
        elif type.startswith('hex'):
            return event.keysym in string.hexdigits
        elif type.startswith(('digit')):
            return event.keysym in string.digits

    def set_value(self):
        pass

    def get_value(self):
        pass

    def set_params(self,*args,**kwargs):
        pass


class NumCtrl(BaseParamWidget):
    """NumCtrl: derived from tk.Entry
    用于输入数值。
    """
    def __init__(self, layout_dir: str, title: str, initial_value: int, unit: str, rang: tuple):
        super().__init__(layout_dir=layout_dir)
        self.on_check_callback = None

        self.prefix = lab_title = QLabel(text=title)
        entryLayout = QHBoxLayout()
        entryLayout.setContentsMargins(0, 0, 0, 0)

        self.ctrl = QLineEdit()
        self.ctrl.textChanged.connect(self.ontext)

        self.postfix = QLabel(text=unit)

        self.central_layout.addWidget(self.prefix)
        self.central_layout.addLayout(entryLayout)
        entryLayout.addWidget(self.ctrl)
        entryLayout.addWidget(self.postfix)

        self.min, self.max = rang
        self.accury = initial_value
        self.set_value(initial_value)

    def Bind(self, z, f):
        self.f = f

    def ontext(self, event):
        self.f(self)
        if self.get_value() == None:
            self.ctrl.setStyleSheet("background-color:#ff0000;")
        else:

            self.ctrl.setStyleSheet("background-color:#ffffff;")
            self.para_changed()
        self.ctrl.update()
        if callable(self.on_check_callback):
            self.on_check_callback()

    def set_value(self, n):
        self.ctrl.clear()
        self.ctrl.setText(str(round(n, self.accury) if self.accury > 0 else int(n)))

    def get_value(self):
        sval = self.ctrl.text()
        try:
            num = float(sval) if self.accury > 0 else int(sval)
        except ValueError:
            import traceback
            traceback.print_exc()
            return None
        if num < self.min or num > self.max:
            return None
        if abs(round(num, self.accury) - num) > 10 ** -(self.accury + 5):  # 这么写才比较严谨吧
            return None
        return num

    def f(self, e):
        pass

    def Refresh(self):
        pass


class TextCtrl(BaseParamWidget):
    def __init__(self, layout_dir: str, title: str, initial_value: str):
        super().__init__(layout_dir)
        self.on_check_callback = None

        self.prefix = lab_title = QLabel(text=title)

        entryLayout = QHBoxLayout()
        entryLayout.setContentsMargins(0, 0, 0, 0)
        self.ctrl = QLineEdit()
        self.ctrl.textChanged.connect(self.ontext)

        # self.postfix = lab_unit = QLabel(text=unit)

        self.central_layout.addWidget(self.prefix)
        self.central_layout.addLayout(entryLayout)
        entryLayout.addWidget(self.ctrl)
        self.set_value(initial_value)
        # entryLayout.addWidget(self.postfix)

    def param_changed(self, event):
        pass

    # ! TODO: what is this?
    def Bind(self, z, f): self.f = f

    def ontext(self, event):
        self.para_changed()

    def set_value(self, text: str):
        self.ctrl.clear()
        self.ctrl.setText(text)

    def get_value(self) -> str:
        return self.ctrl.text()


class ColorCtrl(BaseParamWidget):
    def __init__(self, layout_dir: str, title: str, initial_value: str):
        super().__init__(layout_dir)
        self.on_check_callback = None
        self.prefix = lab_title = QLabel(text=title)

        entryLayout = QHBoxLayout()

        self.ctrl = QLineEdit()
        self.ctrl.textChanged.connect(self.ontext)

        self.color_button = QPushButton()
        self.color_button.clicked.connect(self.oncolor)

        # self.postfix = lab_unit = QLabel(text=unit)
        self.central_layout.addWidget(self.prefix)
        self.central_layout.addLayout(entryLayout)
        entryLayout.addWidget(self.ctrl)
        entryLayout.addWidget(self.color_button)
        # entryLayout.addWidget(self.postfix)
        self.set_value(initial_value)

    def Bind(self, z, f):
        self.f = f

    def ontext(self, event):
        if self.get_value() == None:
            self.color_button.setStyleSheet("background-color:#ff0000;")
            self.ctrl.setStyleSheet("background-color:#ff0000;")
        else:
            self.ctrl.setStyleSheet('background-color:#ffffff;')
            self.color_button.setStyleSheet("background-color:%s;" % self.colorTup2Str(self.get_value()))
            self.para_changed()
        self.ctrl.update()
        if callable(self.on_check_callback):
            self.on_check_callback()

    def oncolor(self, event):
        rst = None
        val = self.get_value()
        color = self.colorTup2Str(self.get_value())
        if color is None:
            color = '#ffffff'
        color = QColorDialog.getColor()  # colorchooser.askcolor(color)  # wx.ColourDialog(self)
        self.set_value(self.colorStr2Tup(color.name()))
        if callable(self.on_check_callback):
            self.on_check_callback()

    def set_value(self, color):
        if color is None:
            color = (255, 255, 255)
        strcolor = self.colorTup2Str(color)
        self.color_button.setStyleSheet('background-color:%s;' % strcolor)
        self.ctrl.clear()
        self.ctrl.setText(strcolor)

    def get_value(self):
        rgb = self.ctrl.text().strip()
        if len(rgb) != 7 or rgb[0] != '#':
            return None
        try:
            int(rgb[1:], 16)
        except:
            import traceback
            traceback.print_exc()
            return None
        return self.colorStr2Tup(rgb)

    def colorStr2Tup(self, value: str) -> tuple:  # pos或者wh的输入都是tuple
        def convert(c):
            v = ord(c)
            if (48 <= v <= 57):
                return v - 48
            else:
                return v - 87  # 返回a的值。

        value = value.lower()
        c0 = convert(value[1])
        c1 = convert(value[2])
        c2 = convert(value[3])
        c3 = convert(value[4])
        c4 = convert(value[5])
        c5 = convert(value[6])
        a1 = c0 * 16 + c1
        a2 = c2 * 16 + c3
        a3 = c4 * 16 + c5
        return (a1, a2, a3)

    def colorTup2Str(self, value: tuple) -> str:
        if value is None:
            return None
        strcolor = '#'
        for i in value:
            strcolor += hex(int(i))[-2:].replace('x', '0')
        return strcolor


class PathCtrl(BaseParamWidget):
    def __init__(self, layout_dir: str, parent, title, filt):
        super().__init__(layout_dir)
        self.prefix = lab_title = QLabel(text=title)
        # self.prefix.pack(fill=tk.X, expand=1)
        path_layout = QHBoxLayout()
        path_layout.addWidget(lab_title)

        # entryFrame = ttk.Frame(self)
        self.ctrl = QLineEdit()  # tk.Entry(entryFrame)
        # self.ctrl.bind('<KeyRelease>', self.ontext)
        # self.ctrl.bind('<ButtonPress-1>', self.onselect)
        path_layout.addWidget(self.ctrl)
        # self.ctrl.pack(side=tk.LEFT, fill=tk.BOTH, expand=True)
        # entryFrame.pack(expand=1, fill=tk.X)
        self.file_choose_button = QPushButton('..')
        path_layout.addWidget(self.file_choose_button)
        self.central_layout.addLayout(path_layout)

    def ontext(self, event):
        self.para_changed()

    def onselect(self, event):
        # [TODO]:对应的这个函数是啥意思？一定要注意一下！
        pass

    def set_value(self, value: str):
        self.ctrl.delete(0, tk.END)
        self.ctrl.insert(0, value)

    def get_value(self) -> str:
        return self.ctrl.get()


class Choice(BaseParamWidget):
    def __init__(self, layout_dir: str, choices, tp, title, unit):
        super().__init__(layout_dir)
        self.tp, self.choices = tp, choices
        self.on_check_callback = None

        self.prefix = lab_title = QLabel(self, text=title)
        self.central_layout.addWidget(self.prefix)
        self.radio_buttons: List['QRadioButton'] = []
        for i, choice in enumerate(self.choices):
            b = QRadioButton(text=str(choice))
            b.toggled.connect(self.on_radio_button_toggled)
            self.radio_buttons.append(b)
            self.central_layout.addWidget(b)

        self.postfix = lab_unit = QLabel(text=unit)
        self.central_layout.addWidget(self.postfix)

    def on_radio_button_toggled(self):
        sender: QRadioButton = self.sender()
        if sender.isChecked():
            index = self.radio_buttons.index(sender)

    def on_choice(self, event=None):
        # attention : button command will not transfer any event as args .
        # 注意：按钮本身并不会传递event作为参数，与键鼠的event不同。
        self.f(self)
        self.para_changed()
        if callable(self.on_check_callback):
            self.on_check_callback()

    def set_value(self, x):
        for i, choice in enumerate(self.choices):
            if x == choice:
                self.radio_buttons[i].setChecked(True)
            else:
                self.radio_buttons[i].setChecked(False)

    def get_value(self):
        for i, radio_button in enumerate(self.radio_buttons):
            if radio_button.isChecked():
                return self.choices[i]


class AnyType(BaseParamWidget):
    def __init__(self, parent, title, types=['Int', 'Float', 'Str']):
        wx.Panel.__init__(self, parent, id=wx.ID_ANY, pos=wx.DefaultPosition, size=wx.Size(-1, -1),
                          style=wx.TAB_TRAVERSAL)

        sizer = wx.BoxSizer(wx.HORIZONTAL)
        self.prefix = lab_title = wx.StaticText(self, wx.ID_ANY, title,
                                                wx.DefaultPosition, wx.DefaultSize)
        lab_title.Wrap(-1)
        sizer.Add(lab_title, 0, wx.ALIGN_CENTER | wx.ALL, 5)

        self.txt_value = wx.TextCtrl(self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0)
        sizer.Add(self.txt_value, 1, wx.ALIGN_CENTER | wx.ALL, 5)
        com_typeChoices = types
        self.postfix = self.com_type = wx.ComboBox(self, wx.ID_ANY, 'Float', wx.DefaultPosition, wx.DefaultSize,
                                                   com_typeChoices, 0)
        sizer.Add(self.com_type, 0, wx.ALIGN_CENTER | wx.ALL, 5)

        self.SetSizer(sizer)
        self.Layout()

        # Connect Events
        self.txt_value.Bind(wx.EVT_KEY_UP, self.on_text)
        self.com_type.Bind(wx.EVT_COMBOBOX, self.on_type)

    def Bind(self, z, f):
        self.f = f

    def set_value(self, v):
        self.txt_value.set_value(str(v))
        if isinstance(v, int):
            self.com_type.Select(0)
        if isinstance(v, float):
            self.com_type.Select(1)
        else:
            self.com_type.Select(2)

    def get_value(self):
        tp = self.com_type.get_value()
        sval = wx.TextCtrl.get_value(self.txt_value)
        if tp == 'Float':
            try:
                num = float(sval)
            except ValueError:
                return None
        if tp == 'Int':
            try:
                num = int(sval)
            except ValueError:
                return None
        if tp == 'Str':
            try:
                num = str(sval)
            except ValueError:
                return None
        return num

    # Virtual event handlers, overide them in your derived class
    def on_text(self, event):
        self.f(self)
        if self.get_value() == None:
            self.txt_value.SetBackgroundColour((255, 255, 0))
        else:
            self.txt_value.SetBackgroundColour((255, 255, 255))
        self.Refresh()

    def on_type(self, event):
        if self.get_value() == None:
            self.txt_value.SetBackgroundColour((255, 255, 0))
        else:
            self.txt_value.SetBackgroundColour((255, 255, 255))
        self.Refresh()


class Choices(BaseParamWidget):
    def __init__(self, parent, choices, title):
        self.choices = list(choices)
        super().__init__(parent)

        self.prefix = lab_title = ttk.Label(self, text=title)
        self.prefix.pack(anchor=tk.W)

        self.boolVarList = [tk.BooleanVar() for i in range(len(self.choices))]
        # print(self.boolVarList)
        self.on_check_callback = None
        for i, choice in enumerate(self.choices):
            b = ttk.Checkbutton(self, text=str(choice),
                                variable=self.boolVarList[i], command=self.on_check)
            b.pack(anchor=tk.W)

    def Bind(self, z, f):
        self.f = f

    def on_check(self):
        self.para_changed()
        if callable(self.on_check_callback):
            self.on_check_callback()

    def get_value(self):
        l = []
        for i, bv in enumerate(self.boolVarList):
            if bv.get() == True:
                l.append(self.choices[i])
        return l

    def set_value(self, value: list):
        for bv in self.boolVarList: bv.set(False)
        for i, v in enumerate(value):
            n = self.choices.index(v) if v in self.choices else -1  # -1 stands for input error
            if n != -1:
                self.boolVarList[n].set(True)
            else:
                warnings.warn('Index item \'%s\' is not in choices list : %s' %
                              (v, repr(self.choices)))


class FloatSlider(BaseParamWidget):
    def __init__(self, parent, rang, accury, title, unit=''):
        super().__init__(parent)
        self.accury = accury
        self.on_check_callback = None

        self.lab_title = ttk.Label(self, text=title)
        self.lab_title.pack(anchor=tk.W)

        self.slider = ttk.Scale(self, from_=rang[0], to=rang[1], orient=tk.HORIZONTAL, command=self.on_scroll)
        self.slider.pack(expand=1, fill=tk.X)
        self.spinboxFrame = ttk.Frame(self)

        self.lab_min = ttk.Label(self.spinboxFrame)
        self.lab_min.pack(side=tk.LEFT)

        self.spinvar = tk.StringVar()
        self.spin = tk.Spinbox(self.spinboxFrame, from_=rang[0], to=rang[1], increment=10 ** -accury, format='%10.4f',
                               command=self.on_spin, textvariable=self.spinvar)
        self.spin.bind('<KeyRelease>', self.on_text)

        self.spin.pack(side=tk.LEFT)
        self.spinboxFrame.pack(expand=1, fill=tk.BOTH, pady=(3, 0))

        self.lab_max = ttk.Label(self.spinboxFrame)
        self.lab_max.pack(side=tk.LEFT)

        self.lab_unit = ttk.Label(self.spinboxFrame, text=unit)
        self.lab_unit.pack(side=tk.LEFT)

        self.set_para(rang, accury)

    def Bind(self, z, f):
        self.f = f

    def set_para(self, rang, accury):
        self.min = round(rang[0], accury)
        self.max = round(rang[1], accury)
        self.lab_min.config(text=str(round(rang[0], accury)))
        self.lab_max.config(text=str(round(rang[1], accury)))
        self.accury = accury

    def on_scroll(self, event):
        '''
        只有这个方法会调用回调函数。原因：当用户通过拖动滚动条进行操作的时候，输入的值一定是合理的，无需进行判断；
        当用户通过按spinbox或者是对spinbox输入文本的时候，只有文本符合规定的时候会对slider设置值，此时会调用这个方法。
        这样总能调用到这个回调函数。
        '''
        value = self.slider.get()
        self.spinvar.set(round(value, self.accury))
        self.spin.config(bg='#ffffff')
        self.para_changed()
        if callable(self.on_check_callback):
            self.on_check_callback()

    def on_spin(self):

        self.slider.set(self.spin.get())
        if callable(self.on_check_callback):
            self.on_check_callback()

    def on_text(self, event):
        if self.is_key(event, 'dir'):
            return
        if self.get_value() == None:
            self.spin.config(bg="#ffff00")
        else:
            self.spin.config(bg="#ffffff")
            self.set_value(self.get_value())

    def set_value(self, n):
        self.slider.set(n)
        self.spinvar.set(round(n, self.accury))

    def get_value(self):
        sval = self.spinvar.get()
        try:
            val = float(sval) if self.accury > 0 else int(float(sval))
            if self.min <= val <= self.max:
                return val
            else:
                return None
        except Exception as e:
            import traceback
            traceback.print_exc()


class Label(BaseParamWidget):
    def __init__(self, parent, title):
        super().__init__(parent)
        lab_title = ttk.Label(self, text=title)
        lab_title.pack(anchor=tk.W)

    def Bind(self, z, f): pass

    def set_value(self, v): pass

    def get_value(self, v): pass


class Check(BaseParamWidget):
    '''
    bool, 'sport', 'do you like sport',True
    '''

    def __init__(self, layout_dir: str, title: str, initial_value: bool):
        super().__init__(layout_dir)
        lab_title = QLabel(text=title)
        layout = QHBoxLayout()
        self.on_check_callback = None
        check = QCheckBox(text=title)
        check.clicked.connect(self.on_check)
        layout.addWidget(check)
        self.check = check
        self.central_layout.addLayout(layout)
        self.set_value(initial_value)

    def get_value(self):
        return self.check.isChecked()

    def set_value(self, value: bool):
        self.check.setChecked(value)

    def on_check(self):
        pass


class ChoiceBoxCtrl(BaseParamWidget):
    def __init__(self, layout_dir: str, title: str, initial_value: object, choices: list, texts=None):

        super().__init__(layout_dir)
        self.choices = []
        self.text_list = []

        lab_title = QLabel(text=title)
        # layout_v = QVBoxLayout()
        layout = QHBoxLayout()
        self.central_layout.addWidget(lab_title)
        # layout_v.addLayout(layout)
        self.on_check_callback = None
        check = QComboBox()

        check.currentIndexChanged.connect(self.on_value_changed)
        layout.addWidget(check)
        self.central_layout.addLayout(layout)
        self.check = check
        # self.central_layout.addLayout(layout_v)
        self.set_choices(choices, texts)
        self.set_value(initial_value)

    def set_choices(self, choices: list, texts: list = None):
        self.check.clear()
        self.choices = choices
        self.text_list = []
        if texts is None:
            for choice in choices:
                self.text_list.append(str(choice))
        else:
            if len(texts) != len(choices):
                raise Exception('Length of argument \'choices\'(len=%d) and \'texts\'(len=%d) are not same!' %
                                (len(choices), len(texts)))
            else:
                self.text_list = texts
        self.check.addItems(self.text_list)

    def on_value_changed(self):
        pass

    def set_value(self, value: object):
        index = self.choices.index(value)
        self.check.setCurrentIndex(index)

    def get_value(self):
        return self.choices[self.check.currentIndex()]


views_dic = {str: TextCtrl, int: NumCtrl, float: NumCtrl, bool: Check, list: Choice}
views_dic.update({'choose_box': ChoiceBoxCtrl, 'chos': Choices, 'color': ColorCtrl, 'number': NumCtrl
                     , 'line_edit': TextCtrl, 'slide': FloatSlider, 'lab': Label, 'bool': Check})


class SettingsPanel(QWidget):
    widgets_dic: Dict[str, BaseParamWidget] = {}

    def __init__(self, parent=None, views: List[Tuple[str]] = None, layout_dir: str = 'v'):
        super(SettingsPanel, self).__init__(parent)
        self.layout_dir = layout_dir
        if layout_dir == 'v':
            self.setLayout(QVBoxLayout())
        else:
            self.setLayout(QHBoxLayout())

        self.set_items(views)

    def _set_items(self, views: List[Tuple[str]] = None):
        if views is None:
            return
        self.widgets_dic: Dict[str, QWidget] = {}
        self.layout().setContentsMargins(0, 0, 0, 0)
        for v in views:
            name = v[1]
            widget = views_dic[v[0]](self.layout_dir, *v[2:])
            if self.widgets_dic.get(name) is None:
                self.widgets_dic[name] = widget
            self.layout().addWidget(widget)
        self.layout().addItem(QSpacerItem(0, 0, QSizePolicy.Minimum, QSizePolicy.Expanding))

    def set_items(self, items: List[Tuple[str]] = None):
        self.widgets_dic = {}
        for i in range(self.layout().count()):
            item = self.layout().itemAt(i).widget()
            if item is not None:
                item.deleteLater()
        self._set_items(items)

    def get_ctrl(self,ctrl_name:str):
        return self.widgets_dic.get(ctrl_name)

    def get_value(self):
        result = {}
        for k in self.widgets_dic:
            result[k] = self.widgets_dic[k].get_value()
        return result

    def closeEvent(self, a0: QCloseEvent) -> None:
        super().closeEvent(a0)
        print(self.get_value())
        self.deleteLater()


if __name__ == '__main__':
    app = QApplication(sys.argv)
    # 类型；名称；显示的提示文字;初始值；//单位；范围
    views = [('line_edit', 'name', 'What\'s your name?', 'hzy'),
             ('number', 'age', 'How old are you?', 88, 'years old', (0, 150)),
             ('number', 'height', 'How High could This Plane fly?', 12000, 'm', (10, 20000)),
             ('bool', 'sport', 'do you like sport', True),
             ('choose_box', 'plane_type', 'plane type', 'f22', ['f22', 'f18', 'j20', 'su57'],
              ['f22战斗机', 'f18战斗轰炸机', 'j20战斗机', 'su57战斗机']),
             ('color', 'color', 'Which color do u like?', (0, 200, 0))]
    sp = SettingsPanel(views=views, layout_dir='v')
    sp.widgets_dic['plane_type'].set_choices(['aaa', 'vvvvv', 'xxxxxx'])
    sp.set_items(views[3:6])
    sp.show()
    sp2 = SettingsPanel(views=views, layout_dir='h')
    sp2.show()
    sp2.setMaximumHeight(30)
    val = sp.get_value()  # 返回一个字典。初始值为表格的第二列：第四列。
    print(val)
    # root.mainloop()
    sys.exit(app.exec_())



import platform
import subprocess


def check_platform() -> str:
    system = platform.system()
    print(system)
    return system.lower()


def run_command_in_terminal(cmd: str, close_mode: str = 'wait_key'):
    platform_name = check_platform()
    if platform_name == 'windows':
        close_action = {'auto': 'start cmd.exe /k \"%s &&exit \"',
                        'no': 'start cmd.exe /k \"%s \"',
                        'wait_key': 'start cmd.exe /k \"%s &&pause &&exit \"'
                        }
        command = close_action[close_mode] % cmd

    elif platform_name == 'deepin':
        command = 'deepin-terminal -x bash -c \" %s \" ' % (cmd)
    elif platform_name == 'linux':
        command = 'gnome-terminal -x bash -c \"%s ;read\"  ' % (cmd)
    else:
        return
    subprocess.Popen(command, shell=True)


if __name__ == '__main__':
    def test_run_in_terminal():
        import time
        run_command_in_terminal('dir', close_mode='no')
        time.sleep(1)
        run_command_in_terminal('dir', close_mode='wait_key')
        time.sleep(1)
        run_command_in_terminal('dir', close_mode='auto')
    test_run_in_terminal()



'''
集成了流式布局、按钮排布的窗口。
'''
from PyQt5.QtWidgets import QScrollArea, QWidget, QToolButton, QVBoxLayout, QSpacerItem, QSizePolicy, QHBoxLayout
from PyQt5.QtCore import Qt, QSize
from typing import TYPE_CHECKING


if TYPE_CHECKING:
    from PyQt5.QtGui import QResizeEvent
    from pmgwidgets import PMFlowLayout


class PMFlowAreaWidget(QWidget):
    def __init__(self):
        super().__init__()
        from pmgwidgets import PMFlowLayout

        self.outer_layout = QVBoxLayout()

        self.flow_layout = PMFlowLayout()
        self.setMinimumWidth(100)
        self.outer_layout.addLayout(self.flow_layout)
        spacer_v = QSpacerItem(20, 20, QSizePolicy.Minimum,
                             QSizePolicy.Expanding)

        self.outer_layout.addItem(spacer_v)
        self.setLayout(self.outer_layout)

    def add_widget(self, w: 'QWidget'):
        self.flow_layout.add_widget(w)

    def setup_ui(self):
        if hasattr(self.widget(), 'setup_ui'):
            self.widget().setup_ui()

    def resizeEvent(self, a0: 'QResizeEvent') -> None:
        super().resizeEvent(a0)
        layout: 'PMFlowLayout' = self.flow_layout
        layout.on_resize()


class PMFlowArea(QScrollArea):
    def __init__(self, parent=None):
        super().__init__(parent)

        self.flow_widget = PMFlowAreaWidget()
        self.widgets_list= self.flow_widget.flow_layout.widgets_list
        self.setWidget(self.flow_widget)
        self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOn)
        self.setWidgetResizable(True)

    def set_layout_content_margins(
            self, left: int, right: int, up: int, down: int):
        self.flow_widget.flow_layout.setContentsMargins(left, right, up, down)

    def add_tool_button(self, name: str, text: str, icon_path: str = ''):
        from pmgwidgets import create_icon
        b = QToolButton()
        b.setText(text)
        icon = create_icon(icon_path)
        b.setIcon(icon)
        b.setToolButtonStyle(Qt.ToolButtonTextUnderIcon)
        b.setIconSize(QSize(40,40))
        b.setMaximumWidth(80)
        b.setMinimumWidth(80)
        b.setMinimumHeight(60)
        b.setMaximumHeight(60)
        self.add_widget(b)
        return b

    def add_widget(self, w: 'QWidget'):
        self.widget().add_widget(w)
        return w

    def setup_ui(self):
        if hasattr(self.widget(), 'setup_ui'):
            self.widget().setup_ui()


if __name__ == '__main__':
    from PyQt5.QtWidgets import QApplication, QPushButton
    import sys

    app = QApplication(sys.argv)
    sa = PMFlowArea()
    for i in range(10):
        w = sa.add_widget(QPushButton('ad%d' % i))
        w.setMaximumHeight(60)
        w.setMinimumHeight(60)
        w.setMinimumWidth(100)
        w.setMaximumWidth(100)
    sa.show()
    sys.exit(app.exec())



from .table import PMGTableWidget, PMTableView, PMGTableTabWidget
from .containers import PMFlowArea, PMScrollArea, PMTabWidget, PMFlowLayout, PMFlowLayoutWithGrid, \
    PMGDockWidget
from .basicwidgets import PMDockObject, PMPushButtonPane, PMToolButton
from .toolbars import PMGToolBar, TopToolBar, ActionWithMessage
from .sourcemgr import create_icon
from .normal import SettingsPanel, center_window, set_always_on_top, set_closable, set_minimizable
from .treeviews import PMGFilesTreeview, RewriteQFileSystemModel
from .display import *
from .platform import *


from PyQt5.QtWidgets import QTabWidget, QWidget


class PMTabWidget(QTabWidget):
    def setup_ui(self):
        for tab_id in range(self.count()):  # 遍历所有的tab
            w = self.widget(tab_id)
            if hasattr(w, 'setup_ui'):
                self.widget(tab_id).setup_ui()

    def addScrolledAreaTab(self, widget: QWidget, a1: str) -> int:
        '''
        添加使用QScrollArea包裹的Tab。
        :param widget:
        :param a1:
        :return:
        '''
        from pmgwidgets import PMScrollArea
        scroll = PMScrollArea()
        scroll.setWidget(widget)

        super().addTab(scroll, a1)



from PyQt5.QtCore import QLineF, QPointF, Qt, QRectF, pyqtSignal
from PyQt5.QtGui import QPolygonF, QPen, QPainterPath, QColor, QPainter, QBrush
from PyQt5.QtWidgets import QGraphicsLineItem, QGraphicsItem, QGraphicsSceneMouseEvent, QGraphicsObject, \
    QStyleOptionGraphicsItem, QWidget

COLOR_NORMAL = QColor(212, 227, 242)
COLOR_HOVER = QColor(255, 200, 00)
COLOR_HOVER_PORT = QColor(0, 0, 50)

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from .flowchart_widget import Node


class CustomLine(QGraphicsLineItem):
    '''
    CustomLine不是QObject，没有办法绑定信号，只能用回调函数的方式。
    '''
    repaint_callback = None

    def __init__(self, start_port: 'CustomPort', end_port: 'CustomPort', canvas: 'PMGraphicsScene' = None):
        super(CustomLine, self).__init__()
        self.setFlags(QGraphicsItem.ItemIsMovable | QGraphicsItem.ItemIsSelectable)  # 拖动
        self.start_port = start_port
        self.end_port = end_port
        self.center_points = [CustomCenterPoint(self)]

        # self.center_point =
        for p in self.center_points:
            canvas.addItem(p)
        # map(canvas.addItem,self.center_points)
        center_point = self.center_points[0]
        center_point.setPos(self.start_port.x(), self.end_port.y())
        center_point.point_dragged.connect(self.refresh)

        self.refresh()
        self.canvas = canvas

    def refresh(self):
        self.setLine(QLineF(self.start_port.center_pos, self.end_port.center_pos))
        self.update()

    def mousePressEvent(self, event: 'QGraphicsSceneMouseEvent') -> None:
        print('mouse clicked!!')

    def draw_arrow(self, QPainter, point_1: QPointF, point_2: QPointF) -> 'QPolygonF':

        l = QLineF(point_1, point_2)
        v = l.unitVector()

        v.setLength(20)  # 改变单位向量的大小，实际就是改变箭头长度
        v.translate(QPointF(int(l.dx() / 2), int(l.dy() / 2)))

        n = v.normalVector()  # 法向量
        n.setLength(n.length() * 0.2)  # 这里设定箭头的宽度
        n2 = n.normalVector().normalVector()  # 两次法向量运算以后，就得到一个反向的法向量
        p1 = v.p2()
        p2 = n.p2()
        p3 = n2.p2()
        QPainter.drawPolygon(p1, p2, p3)
        return QPolygonF([p1, p2, p3, p1])

    def paint(self, q_painter:'QPainter', style_option_graphics_item:'QStyleOptionGraphicsItem', widget:'QWidget'=None):
        # setPen
        pen = QPen()
        pen.setWidth(5)
        pen.setJoinStyle(Qt.MiterJoin)  # 让箭头变尖
        q_painter.setPen(pen)

        path = QPainterPath()

        point1 = self.start_port
        path.moveTo(self.start_port.center_pos)
        for p in self.center_points + [self.end_port]:
            q_painter.drawLine(QLineF(point1.center_pos, p.center_pos))
            arrow = self.draw_arrow(q_painter, point1.center_pos, p.center_pos)
            path.addPolygon(arrow)
            point1 = p

        if self.repaint_callback is not None:
            self.repaint_callback()


class CustomCenterPoint(QGraphicsObject):
    point_dragged = pyqtSignal(QGraphicsObject)

    def __init__(self, line: 'CustomLine'):
        super(CustomCenterPoint, self).__init__()
        # self.setFlags(QGraphicsItem.ItemIsMovable | QGraphicsItem.ItemIsSelectable)  # 拖动
        self.setAcceptHoverEvents(True)  # 接受鼠标悬停事件
        self.relative_pos = (0, 0)
        self.color = COLOR_NORMAL
        self.size = (10, 10)
        self.line = line

    def boundingRect(self):
        return QRectF(0, 0, self.size[0], self.size[1])

    @property
    def center_pos(self):
        return QPointF(self.x() + self.size[0] / 2, self.y() + self.size[1] / 2)

    def mouseMoveEvent(self, event: 'QGraphicsSceneMouseEvent') -> None:
        mouse_x, mouse_y = event.scenePos().x(), event.scenePos().y()
        self.setPos(QPointF(mouse_x, mouse_y))
        self.point_dragged.emit(self)
        print('dr')

    def paint(self, painter, styles, widget=None):
        pen1 = QPen(Qt.SolidLine)
        pen1.setColor(QColor(128, 128, 128))
        painter.setPen(pen1)

        brush1 = QBrush(Qt.SolidPattern)
        brush1.setColor(self.color)
        painter.setBrush(brush1)

        painter.setRenderHint(QPainter.Antialiasing)  # 反锯齿
        painter.drawRoundedRect(self.boundingRect(), 10, 10)

    def hoverEnterEvent(self, event: 'QGraphicsSceneHoverEvent') -> None:
        self.color = COLOR_HOVER_PORT

        self.update()

    def hoverLeaveEvent(self, event: 'QGraphicsSceneHoverEvent') -> None:
        self.color = COLOR_NORMAL
        self.update()

    def mousePressEvent(self, evt: QGraphicsSceneMouseEvent):
        print('clicked on port')
        if evt.button() == Qt.LeftButton:
            print("左键被按下")
            pos = (evt.scenePos().x(), evt.scenePos().y())
            # print(self.x(), type(self.x()), type(pos[1]))
            self.relative_pos = (pos[0] - self.x(), pos[1] - self.y())

        elif evt.button() == Qt.RightButton:
            print("左键被按下")
        elif evt.button() == Qt.MidButton:
            print("中间键被按下")

    def paintEvent(self, QPaintEvent):
        pen1 = QPen()
        pen1.setColor(QColor(166, 66, 250))
        painter = QPainter(self)
        painter.setPen(pen1)
        painter.begin(self)
        painter.drawRoundedRect(self.boundingRect(), 10, 10)  # 绘制函数
        painter.end()


class CustomPort(QGraphicsObject):
    port_clicked = pyqtSignal(QGraphicsObject)

    def __init__(self, port_id: int):
        super(CustomPort, self).__init__()
        self.setAcceptHoverEvents(True)  # 接受鼠标悬停事件
        self.relative_pos = (0, 0)
        self.color = COLOR_NORMAL
        self.id = port_id
        self.size = (10, 10)

    def boundingRect(self):
        return QRectF(0, 0, self.size[0], self.size[1])

    @property
    def center_pos(self):
        return QPointF(self.x() + self.size[0] / 2, self.y() + self.size[1] / 2)

    def paint(self, painter, styles, widget=None):
        pen1 = QPen(Qt.SolidLine)
        pen1.setColor(QColor(128, 128, 128))
        painter.setPen(pen1)

        brush1 = QBrush(Qt.SolidPattern)
        brush1.setColor(self.color)
        painter.setBrush(brush1)

        painter.setRenderHint(QPainter.Antialiasing)  # 反锯齿
        painter.drawRoundedRect(self.boundingRect(), 10, 10)

    def hoverEnterEvent(self, event: 'QGraphicsSceneHoverEvent') -> None:
        self.color = COLOR_HOVER_PORT

        self.update()

    def hoverLeaveEvent(self, event: 'QGraphicsSceneHoverEvent') -> None:
        self.color = COLOR_NORMAL
        self.update()

    def mousePressEvent(self, evt: QGraphicsSceneMouseEvent):
        print('clicked on port')
        if evt.button() == Qt.LeftButton:
            print("左键被按下")
            pos = (evt.scenePos().x(), evt.scenePos().y())
            self.relative_pos = (pos[0] - self.x(), pos[1] - self.y())
            # print(self.relative_pos)
            self.port_clicked.emit(self)
        elif evt.button() == Qt.RightButton:
            print("左键被按下")
        elif evt.button() == Qt.MidButton:
            print("中间键被按下")

    def paintEvent(self, QPaintEvent):
        pen1 = QPen()
        pen1.setColor(QColor(166, 66, 250))
        painter = QPainter(self)
        painter.setPen(pen1)
        painter.begin(self)
        painter.drawRoundedRect(self.boundingRect(), 10, 10)  # 绘制函数
        painter.end()


class CustomRect(QGraphicsObject):
    def __init__(self, node: 'Node' = None):
        super(CustomRect, self).__init__()
        # self.setFlags(QGraphicsItem.ItemIsMovable | QGraphicsItem.ItemIsSelectable)  # 拖动
        self.setAcceptHoverEvents(True)  # 接受鼠标悬停事件
        self.relative_pos = (0, 0)
        self.color = COLOR_NORMAL
        self.node = node

    def boundingRect(self):
        return QRectF(0, 0, 200, 50)

    def paint(self, painter, styles, widget=None):
        pen1 = QPen(Qt.SolidLine)
        pen1.setColor(QColor(128, 128, 128))
        painter.setPen(pen1)

        brush1 = QBrush(Qt.SolidPattern)
        brush1.setColor(self.color)
        painter.setBrush(brush1)

        painter.setRenderHint(QPainter.Antialiasing)  # 反锯齿
        painter.drawRoundedRect(self.boundingRect(), 10, 10)

    def hoverEnterEvent(self, event: 'QGraphicsSceneHoverEvent') -> None:
        self.color = COLOR_HOVER
        self.update()
        print('enter')

    def hoverLeaveEvent(self, event: 'QGraphicsSceneHoverEvent') -> None:
        self.color = COLOR_NORMAL
        self.update()

    def mouseMoveEvent(self, event: QGraphicsSceneMouseEvent) -> None:
        self.setPos(event.scenePos().x() - self.relative_pos[0], event.scenePos().y() - self.relative_pos[1])
        self.node.refresh_pos()

    def mousePressEvent(self, evt: QGraphicsSceneMouseEvent):
        print('鼠标按下')
        if evt.button() == Qt.LeftButton:
            print("左键被按下")
            pos = (evt.scenePos().x(), evt.scenePos().y())
            print(self.x(), type(self.x()), type(pos[1]))
            self.relative_pos = (pos[0] - self.x(), pos[1] - self.y())
            print(self.relative_pos)
        elif evt.button() == Qt.RightButton:
            print("左键被按下")
        elif evt.button() == Qt.MidButton:
            print("中间键被按下")

    def paintEvent(self, QPaintEvent):
        pen1 = QPen()
        pen1.setColor(QColor(166, 66, 250))
        painter = QPainter(self)
        painter.setPen(pen1)
        painter.begin(self)
        painter.drawRoundedRect(self.boundingRect(), 10, 10)  # 绘制函数
        painter.end()



from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QWidget, QDesktopWidget


def center_window(window: QWidget):
    screen = QDesktopWidget().screenGeometry()
    size = window.geometry()
    window.move(int((screen.width() - size.width()) / 2),
                int((screen.height() - size.height()) / 2))


def set_always_on_top(window: QWidget):
    flags = window.windowFlags()
    window.setWindowFlags(flags | Qt.WindowStaysOnTopHint)  # 窗体总在最前端


def set_minimizable(window: QWidget):
    flags = window.windowFlags()
    window.setWindowFlags(flags | Qt.WindowMinMaxButtonsHint)

def set_closable(window:QWidget):
    flags = window.windowFlags()
    window.setWindowFlags(flags | Qt.WindowCloseButtonHint)


from .PMTab import PMTabWidget
from .pmscrollarea import PMScrollArea
from .flowarea import PMFlowArea
from .flowlayout import PMFlowLayout, PMFlowLayoutWithGrid
from .pmdockwidget import PMGDockWidget
from .pmtoolbox import PMGToolBox



from typing import List, Callable

from PyQt5.QtCore import Qt, QSize
from PyQt5.QtGui import QIcon
from PyQt5.QtWidgets import QToolBar, QPushButton, QMenu, QToolButton, QAction, QWidget, QLabel, QSizePolicy


class ActionWithMessage(QAction):
    def __init__(self, text: str = '', icon: QIcon = None,
                 parent: QWidget = None, message: str = ''):
        super().__init__(parent)
        self.setText(text)
        if icon is not None:
            self.setIcon(icon)
        self.message = message


class TopToolBar(QToolBar):
    def __init__(self):
        super().__init__()
        self.setFloatable(False)
        self.setLayoutDirection(Qt.LeftToRight)
        self.setMovable(False)
        self.setContentsMargins(0,0,0,0)

    def add_button(self, text: str):
        b = QPushButton(text)
        b.setObjectName('pmtopToolbarButton')
        b.setProperty('stat', 'unselected')
        self.addWidget(b)
        self.addWidget(QLabel('  '))  # 增加一段空间。
        return b

    def get_button(self, name: str):
        return self.findChild(QPushButton, name)


class TopToolBarRight(QToolBar):
    def __init__(self):
        super().__init__()
        self.setFloatable(False)
        self.setMovable(False)

        self.setContentsMargins(0, 0, 0, 0)
        self.setLayoutDirection(Qt.RightToLeft)
        self.hide_button = QToolButton()
        self.hide_button.setMaximumWidth(15)
        self.hide_button.setArrowType(Qt.UpArrow)
        self.addWidget(self.hide_button)



class PMGToolBar(QToolBar):
    tab_button: QPushButton = None
    _control_widget_dic = {}

    def __init__(self,parent=None):
        super().__init__(parent)

        self.setFixedHeight(100)
        self.setMinimumHeight(100)
        self.setMaximumHeight(100)
        self._control_widget_dic = {}
        self.add_height_occupation()
    def get_control_widget(self, widget_name: str) -> QPushButton:
        w = self._control_widget_dic.get(widget_name)
        if w is None:
            raise Exception(
                'Toolbar \'%s\' has no widget named \'%s\'' %
                widget_name)
        return w

    def add_tool_button(self, name: str, text: str = '',
                        icon: QIcon = None, menu: QMenu = None):
        tb = QToolButton()
        tb.setPopupMode(QToolButton.InstantPopup)
        tb.setToolButtonStyle(Qt.ToolButtonTextUnderIcon)
        tb.setText(text)

        tb.setStyleSheet('QToolButton{height:60px;width:60px;border:0px;}')
        if icon is not None:
            tb.setIcon(icon)
            tb.setIconSize(QSize(40, 40))
        if menu is not None:
            tb.setMenu(menu)
        self.addWidget(tb)
        self._control_widget_dic[name] = tb
        return tb
    def add_height_occupation(self):
        button_num = 3
        texts=['','','']
        from pmgwidgets import PMPushButtonPane

        pp = PMPushButtonPane()
        button_list = pp.add_height_occu_buttons()
        self.addWidget(pp)
        return button_list

    def add_buttons(self, button_num: int, names: List[str], texts: List[str], icons_path: List[str] = None) -> List[
        'QPushButton']:
        from pmgwidgets import PMPushButtonPane

        pp = PMPushButtonPane()
        button_list = pp.add_buttons(button_num, texts, icons_path)
        for i, name in enumerate(names):
            self._control_widget_dic[name] = button_list[i]
        self.addWidget(pp)
        return button_list

    def add_widget(self, name: str, widget: 'QWidget'):
        self._control_widget_dic[name] = widget
        self.addWidget(widget)
        return widget

    def add_menu_to(self, button_name: str,
                    action_texts: List[str], action_commands: List['Callable'])-> None:
        button = self.get_control_widget(button_name)
        if button is not None:
            menu = QMenu(self)
            for text, cmd in zip(action_texts, action_commands):
                a = QAction(text=text, parent=menu)
                menu.addAction(a)
                a.triggered.connect(cmd)
            button.setMenu(menu)

    def append_menu(self,button_name:str,action_text:str,action_command:'Callable')->'QAction':
        button:'QPushButton' = self.get_control_widget(button_name)
        if button is not None:
            menu = button.menu()
            if menu is None:
                self.add_menu_to(button_name,[action_text],[action_command])
                return

            a = QAction(text=action_text, parent=menu)
            menu.addAction(a)
            print(menu,action_text)
            a.triggered.connect(action_command)
            return a
        return None


import sys

from PyQt5.QtWidgets import QWidget, QVBoxLayout, QHBoxLayout, QToolButton, QSpacerItem, QSizePolicy, QGraphicsView, \
    QFrame, QApplication, QGraphicsObject, QGraphicsScene, QGraphicsSceneMouseEvent
from PyQt5.QtCore import QSize, QCoreApplication, QRectF, Qt, QPointF, pyqtSignal, QLineF, QObject
from PyQt5.QtGui import QIcon, QPixmap, QPen, QColor, QBrush, QPainter, QPainterPath

from pmgwidgets.flowchart.flow_items import CustomRect, CustomPort

COLOR_NORMAL = QColor(212, 227, 242)
COLOR_HOVER = QColor(255, 200, 00)
COLOR_HOVER_PORT = QColor(0, 0, 50)


class PMGraphicsScene(QGraphicsScene):
    signal_item_dragged = pyqtSignal(str)
    signal_port_clicked = pyqtSignal(str)

    def __init__(self, parent=None, graphics_view: 'QGraphicsView' = None):
        super().__init__(parent)
        self.drawing_lines = False
        self.line_start_port = None
        self.line_start_point = None
        self.line_end_port = None
        self.line = self.addLine(0, 0, 100, 100, QPen())
        self.graphics_view = graphics_view

    def mouseMoveEvent(self, event: 'QGraphicsSceneMouseEvent') -> None:
        super(PMGraphicsScene, self).mouseMoveEvent(event)
        if self.drawing_lines == True:
            self.line.show()
            self.line_end_point = event.scenePos()
            self.line.setLine(QLineF(self.line_end_point, self.line_start_point))

    def connect_port(self, end_port):
        from pmgwidgets.flowchart.flow_items import CustomLine
        self.line.hide()
        line = CustomLine(self.line_start_port, end_port, self)
        line.repaint_callback = lambda: self.graphics_view.viewport().update()
        self.addItem(line)
        self.signal_item_dragged.connect(line.refresh)
        self.drawing_lines = False


class Node(QObject):
    def __init__(self, canvas: 'PMGraphicsScene'):
        super(Node, self).__init__()
        self.base_rect = CustomRect(self)
        self.input_ports = [CustomPort(0), CustomPort(1)]
        self.output_ports = [CustomPort(2), CustomPort(3)]
        self.canvas = canvas
        self.setup()

    def refresh_pos(self):
        y = self.base_rect.y()
        dy_input = self.base_rect.boundingRect().height() / (1 + len(self.input_ports))
        dy_output = self.base_rect.boundingRect().height() / (1 + len(self.output_ports))
        x_input = self.base_rect.x() + 5
        x_output = self.base_rect.x() + self.base_rect.boundingRect().width() - 15
        for i, p in enumerate(self.input_ports):
            p.setPos(x_input, y + int(dy_input * (1 + i)))
        for i, p in enumerate(self.output_ports):
            p.setPos(x_output, y + int(dy_output * (1 + i)))
        self.canvas.signal_item_dragged.emit('')

    def setup(self):
        self.base_rect.setPos(80, 80)
        self.refresh_pos()

        self.canvas.addItem(self.base_rect)
        for p in self.input_ports + self.output_ports:
            self.canvas.addItem(p)
            p.port_clicked.connect(self.on_port_clicked)

    def on_port_clicked(self, port: 'CustomPort'):
        if self.canvas.drawing_lines == True:
            if not self.canvas.line_start_port is port:
                self.canvas.connect_port(port)
            else:
                self.canvas.drawing_lines = False
        else:
            self.canvas.drawing_lines = True
            self.canvas.line_start_point = port.center_pos
            self.canvas.line_start_port = port


class PMFlowWidget(QWidget):
    def __init__(self):
        # from pyminer2.ui.base.widgets.resources import icon_lc_save, icon_sc_shadowcurser, \
        #     icon_lc_connectorlinesarrowend, icon_lc_undo, \
        #     icon_lc_redo, icon_sc_deletepage, icon_lc_aligncenter, icon_lc_alignmiddle, icon_lc_zoomin, icon_lc_zoomout, \
        #     icon_dataprovider, icon_sc_autosum, icon_lc_drawchart, icon_lc_statisticsmenu, icon_lc_dbreportedit
        _translate = QCoreApplication.translate
        super().__init__()
        self.setObjectName("tab_flow")

        self.verticalLayout_6 = QVBoxLayout(self)
        self.verticalLayout_6.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_6.setSpacing(0)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.widget_3 = QWidget(self)
        self.widget_3.setMinimumSize(QSize(0, 30))
        self.widget_3.setObjectName("widget_3")

        self.horizontalLayout_6 = QHBoxLayout(self.widget_3)
        self.horizontalLayout_6.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_6.setSpacing(1)
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.toolButton_4 = QToolButton(self.widget_3)
        icon7 = QIcon()
        icon7.addPixmap(QPixmap(":/pyqt/source/images/NavOverFlow_Start.png"), QIcon.Normal,
                        QIcon.Off)
        self.toolButton_4.setIcon(icon7)
        self.toolButton_4.setIconSize(QSize(25, 25))
        self.toolButton_4.setObjectName("toolButton_4")
        self.horizontalLayout_6.addWidget(self.toolButton_4)
        self.toolButton_3 = QToolButton(self.widget_3)
        icon8 = QIcon()
        icon8.addPixmap(QPixmap(":/pyqt/source/images/lc_basicstop.png"), QIcon.Normal, QIcon.Off)
        self.toolButton_3.setIcon(icon8)
        self.toolButton_3.setIconSize(QSize(25, 25))
        self.toolButton_3.setObjectName("toolButton_3")
        self.horizontalLayout_6.addWidget(self.toolButton_3)
        self.toolButton_6 = QToolButton(self.widget_3)

        # self.toolButton_6.setIcon(icon_lc_save)
        self.toolButton_6.setIconSize(QSize(25, 25))
        self.toolButton_6.setObjectName("toolButton_6")
        self.horizontalLayout_6.addWidget(self.toolButton_6)
        self.toolButton_5 = QToolButton(self.widget_3)

        # self.toolButton_5.setIcon(icon_sc_shadowcurser)
        self.toolButton_5.setIconSize(QSize(25, 25))
        self.toolButton_5.setObjectName("toolButton_5")
        self.horizontalLayout_6.addWidget(self.toolButton_5)
        self.toolButton_2 = QToolButton(self.widget_3)

        # self.toolButton_2.setIcon(icon_lc_connectorlinesarrowend)
        self.toolButton_2.setIconSize(QSize(25, 25))
        self.toolButton_2.setObjectName("toolButton_2")
        self.horizontalLayout_6.addWidget(self.toolButton_2)
        self.toolButton = QToolButton(self.widget_3)

        # self.toolButton.setIcon(icon_lc_undo)
        self.toolButton.setIconSize(QSize(25, 25))
        self.toolButton.setObjectName("toolButton")
        self.horizontalLayout_6.addWidget(self.toolButton)
        self.toolButton_7 = QToolButton(self.widget_3)

        # self.toolButton_7.setIcon(icon_lc_redo)
        self.toolButton_7.setIconSize(QSize(25, 25))
        self.toolButton_7.setObjectName("toolButton_7")
        self.horizontalLayout_6.addWidget(self.toolButton_7)
        self.toolButton_8 = QToolButton(self.widget_3)

        # self.toolButton_8.setIcon(icon_sc_deletepage)
        self.toolButton_8.setIconSize(QSize(25, 25))
        self.toolButton_8.setObjectName("toolButton_8")
        self.horizontalLayout_6.addWidget(self.toolButton_8)
        self.toolButton_9 = QToolButton(self.widget_3)

        # self.toolButton_9.setIcon(icon_lc_aligncenter)
        self.toolButton_9.setIconSize(QSize(25, 25))
        self.toolButton_9.setObjectName("toolButton_9")
        self.horizontalLayout_6.addWidget(self.toolButton_9)
        self.toolButton_10 = QToolButton(self.widget_3)

        # self.toolButton_10.setIcon(icon_lc_alignmiddle)
        self.toolButton_10.setIconSize(QSize(25, 25))
        self.toolButton_10.setObjectName("toolButton_10")
        self.horizontalLayout_6.addWidget(self.toolButton_10)
        self.toolButton_11 = QToolButton(self.widget_3)

        # self.toolButton_11.setIcon(icon_lc_zoomin)
        self.toolButton_11.setIconSize(QSize(25, 25))
        self.toolButton_11.setObjectName("toolButton_11")
        self.horizontalLayout_6.addWidget(self.toolButton_11)
        self.toolButton_12 = QToolButton(self.widget_3)

        # self.toolButton_12.setIcon(icon_lc_zoomout)
        self.toolButton_12.setIconSize(QSize(25, 25))
        self.toolButton_12.setObjectName("toolButton_12")
        self.horizontalLayout_6.addWidget(self.toolButton_12)
        self.toolButton_13 = QToolButton(self.widget_3)

        # self.toolButton_13.setIcon(icon_dataprovider)
        self.toolButton_13.setIconSize(QSize(25, 25))
        self.toolButton_13.setObjectName("toolButton_13")
        self.horizontalLayout_6.addWidget(self.toolButton_13)
        self.toolButton_16 = QToolButton(self.widget_3)

        # self.toolButton_16.setIcon(icon_sc_autosum)
        self.toolButton_16.setIconSize(QSize(25, 25))
        self.toolButton_16.setObjectName("toolButton_16")
        self.horizontalLayout_6.addWidget(self.toolButton_16)
        self.toolButton_17 = QToolButton(self.widget_3)

        # self.toolButton_17.setIcon(icon_lc_drawchart)
        self.toolButton_17.setIconSize(QSize(25, 25))
        self.toolButton_17.setObjectName("toolButton_17")
        self.horizontalLayout_6.addWidget(self.toolButton_17)
        self.toolButton_14 = QToolButton(self.widget_3)

        # self.toolButton_14.setIcon(icon_lc_statisticsmenu)
        self.toolButton_14.setIconSize(QSize(25, 25))
        self.toolButton_14.setObjectName("toolButton_14")
        self.horizontalLayout_6.addWidget(self.toolButton_14)
        self.toolButton_15 = QToolButton(self.widget_3)

        # self.toolButton_15.setIcon(icon_lc_dbreportedit)
        self.toolButton_15.setIconSize(QSize(25, 25))
        self.toolButton_15.setObjectName("toolButton_15")
        self.horizontalLayout_6.addWidget(self.toolButton_15)
        spacerItem = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_6.addItem(spacerItem)
        self.verticalLayout_6.addWidget(self.widget_3)

        self.graphicsView = QGraphicsView(self)

        self.graphicsView.setFrameShape(QFrame.NoFrame)
        self.graphicsView.setObjectName("graphicsView")
        self.verticalLayout_6.addWidget(self.graphicsView)

        self.toolButton_4.setText(_translate("MainWindow", "..."))
        self.toolButton_3.setText(_translate("MainWindow", "..."))
        self.toolButton_6.setText(_translate("MainWindow", "..."))
        self.toolButton_5.setText(_translate("MainWindow", "..."))
        self.toolButton_2.setText(_translate("MainWindow", "..."))
        self.toolButton.setText(_translate("MainWindow", "..."))
        self.toolButton_7.setText(_translate("MainWindow", "..."))
        self.toolButton_8.setText(_translate("MainWindow", "..."))
        self.toolButton_9.setText(_translate("MainWindow", "..."))
        self.toolButton_10.setText(_translate("MainWindow", "..."))
        self.toolButton_11.setText(_translate("MainWindow", "..."))
        self.toolButton_12.setText(_translate("MainWindow", "..."))
        self.toolButton_13.setText(_translate("MainWindow", "..."))
        self.toolButton_16.setText(_translate("MainWindow", "..."))
        self.toolButton_17.setText(_translate("MainWindow", "..."))
        self.toolButton_14.setText(_translate("MainWindow", "..."))
        self.toolButton_15.setText(_translate("MainWindow", "..."))

        # self.rect = CustomRect()
        # self.rect.setPos(50, 50)
        # self.rect2 = CustomRect()
        # self.rect2.setPos(100, 100)
        #
        self.scene = PMGraphicsScene(graphics_view=self.graphicsView)
        self.scene.setSceneRect(0, 0, 300, 300)
        # self.scene.addItem(self.rect)
        # self.scene.addItem(self.rect2)
        #
        # self.cl = CustomLine()
        # self.cl.start_pos = (0, 30)
        # self.cl.end_pos = (100, 100)
        # self.scene.addItem(self.cl)
        self.n = Node(self.scene)
        self.n2 = Node(self.scene)

        self.graphicsView.setScene(self.scene)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    graphics = PMFlowWidget()
    graphics.show()

    sys.exit(app.exec_())



'''
代码来源：
https://www.cnblogs.com/taostaryu/p/9772492.html

'''
import sys

from PyQt5.QtCore import QUrl
from PyQt5.QtWebEngineWidgets import QWebEngineView
from PyQt5.QtWidgets import QMainWindow, QApplication


class MainWindow(QMainWindow):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.setWindowTitle('My Browser')
        self.showMaximized()

        self.webview = WebEngineView()
        # self.webview.load(QUrl("https://cn.bing.com"))
        self.webview.load(QUrl("show_formula.html"))
        self.setCentralWidget(self.webview)


class WebEngineView(QWebEngineView):
    windowList = []

    # 重写createwindow()
    def createWindow(self, QWebEnginePage_WebWindowType):
        new_webview = WebEngineView()
        new_window = MainWindow()
        new_window.setCentralWidget(new_webview)
        # new_window.show()
        self.windowList.append(new_window)  # 注：没有这句会崩溃！！！
        return new_webview


if __name__ == "__main__":
    app = QApplication(sys.argv)
    w = MainWindow()
    w.show()
    sys.exit(app.exec_())



from PyQt5.QtWidgets import QToolBox, QWidget, QToolButton
from typing import Dict, Callable, List


class PMGToolBox(QToolBox):
    group_widgets: Dict[str, QWidget] = {}

    def __init__(self, parent=None):
        super().__init__(parent)
        self.button_num = 0
        pass

    def set_group_text(self, group_name: str, text: str):
        gw = self.group_widgets.get(group_name)
        if gw is not None:
            self.setItemText(self.indexOf(gw), text)

    def add_button(self, group_name: str, text: str, icon_path: str, action: Callable):
        from pmgwidgets import create_icon
        from pmgwidgets import PMFlowArea
        if self.group_widgets.get(group_name) is None:
            fa = PMFlowArea()
            self.group_widgets[group_name] = fa
            self.addItem(fa, group_name)
            btn = fa.add_tool_button(name='button#%d' % self.button_num, text=text, icon_path=icon_path)
            btn.clicked.connect(action)
            self.button_num += 1
        else:
            fa: PMFlowArea = self.group_widgets[group_name]
            btn = fa.add_tool_button(name='button#%d' % self.button_num, text=text, icon_path=icon_path)
            btn.clicked.connect(action)
            self.button_num += 1



import os


def add_translation_file(file_path:str):
    from PyQt5.QtWidgets import QApplication
    from PyQt5.QtCore import QLocale, QTranslator
    app = QApplication.instance()
    if hasattr(app, 'trans'):
        try:
            tr = QTranslator()
            path = file_path
            tr.load(path)
            app.installTranslator(tr)
        except:
            pass



from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg
import matplotlib.pyplot as plt
from PyQt5.QtWidgets import QVBoxLayout, QWidget, QApplication
import sys
from typing import TYPE_CHECKING
if TYPE_CHECKING:
    from matplotlib.axes._subplots import Axes


class PMMatplotlibQt5Widget(QWidget):
    def __init__(self,parent=None):
        super().__init__(parent)
        self.figure = plt.figure(facecolor='#FFD7C4')  # 可选参数,facecolor为背景颜色
        self.canvas = FigureCanvasQTAgg(self.figure)
        layout = QVBoxLayout()
        layout.addWidget(self.canvas)
        self.setLayout(layout)

    def add_subplot(self, param) -> 'Axes':
        return self.figure.add_subplot(param)

    def draw(self)->None:
        self.canvas.draw()



'''
这是一个利用QT的MVC架构进行数据查看的表格。这个表格十分适合大量数据的查看，1000*1000规模的数据集可以做到秒开。
其中定义了若干类。可以直接显示pd.DataFrame,np.array和list的TableView。
'''
import sys
import time

import typing
from PyQt5.QtWidgets import QTableView, QApplication
from PyQt5.QtCore import QAbstractTableModel
from PyQt5.QtCore import Qt
import numpy as np


def to_decimal_str(cell_data: 'np.ndarray', decimals: int = 6):
    try:
        rounded_data = np.around(cell_data, decimals)
        return repr(rounded_data)
    except:
        return repr(cell_data)


def dataformat(val, decimals=6, sci=False):
    '''
    这只是暂时的strformat函数。如有可能，应当使用cython重写并且部署在动态链接库中,从而提升性能。
    Args:
        val:
        decimals:
        sci:

    Returns:

    '''
    global type_float_set
    return to_decimal_str(val, decimals)
    value_str = repr(val)

    if hasattr(val, 'dtype'):
        if np.issubdtype(val, np.inexact):
            if not sci:
                return str(np.around(val, decimals=decimals))
            else:
                return '%e' % np.around(val, decimals=decimals)
    elif isinstance(val, int):
        if sci:
            return '%e' % val
        return str(val)
    elif isinstance(val, float):
        if sci:
            return '%e' % round(val, decimals)
        return str(val)
    elif isinstance(val, complex):
        if sci:
            return '%e+%ej' % (round(val.real), round(val.imag))
        else:
            return '%f+%fj' % (round(val.real), round(val.imag))
    else:
        return str(val)


class TableModelForList(QAbstractTableModel):
    '''
    输入为list的table model
    '''

    def __init__(self, data: list):
        super(TableModelForList, self).__init__()
        self._data = data

    def data(self, index, role):
        if role == Qt.DisplayRole:
            return dataformat(self._data[index.row()][index.column()])

    def rowCount(self, index):
        return len(self._data)

    def columnCount(self, index):
        return len(self._data[0])


class TableModelForNumpyArray(QAbstractTableModel):
    '''
    输入为pandas.DataFram的TableModel，用于在表格中显示数据。
    '''

    def __init__(self, data):
        super(QAbstractTableModel, self).__init__()
        self._data = data

    def data(self, index, role):
        if role == Qt.DisplayRole:
            if len(self._data.shape) >= 2:
                value = self._data[index.row(), index.column()]
            else:
                value = self._data[index.row()]
            return dataformat(value)

    def rowCount(self, index):
        return self._data.shape[0]

    def columnCount(self, index):
        if len(self._data.shape) == 1:
            return 1
        else:
            return self._data.shape[1]


class TableModelForPandasDataframe(QAbstractTableModel):
    '''
    输入为pandas.DataFram的TableModel，用于在表格中显示数据。
    '''

    def __init__(self, data):
        super(QAbstractTableModel, self).__init__()
        self._data = data

    def data(self, index, role):
        if role == Qt.DisplayRole:
            value = self._data.iloc[index.row(), index.column()]
            return dataformat(value)

    def rowCount(self, index):
        return self._data.shape[0]

    def columnCount(self, index):
        return self._data.shape[1]

    def headerData(self, section: int, orientation: Qt.Orientation, role: int = ...) -> typing.Any:
        if role == Qt.DisplayRole:
            if role == Qt.DisplayRole:
                if orientation == Qt.Horizontal:
                    return str(self._data.columns[section])
                if orientation == Qt.Vertical:
                    return str(self._data.index[section])


class PMTableView(QTableView):
    '''
    基类，用于显示数据。输入数据类型为列表。
    '''

    def __init__(self, data=None):
        super().__init__()
        if data is not None:
            self.set_data(data)

    def set_data(self, data):
        import pandas as pd
        import numpy as np
        if isinstance(data, pd.DataFrame):
            self.model = TableModelForPandasDataframe(data)
        elif isinstance(data, np.ndarray):
            self.model = TableModelForNumpyArray(data)
        elif isinstance(data, list):
            self.model = TableModelForList(data)
        else:
            raise Exception("data type %s is not supported in PMTableView.\
                            \n Supported Types are: numpy.array,list and pandas.DataFrame." % type(data))
        self.setModel(self.model)


if __name__ == '__main__':
    import pandas as pd

    app = QApplication(sys.argv)
    table = PMTableView()
    import numpy as np

    data = pd.DataFrame(np.random.random((1000, 1000)), columns=['a' + str(i) for i in range(1000)],
                        index=['row' + str(i) for i in range(1000)])
    table.show()
    table.set_data(data.values)

    app.exec_()



from typing import List


def create_translation(target_files: str):
    import os
    folder = os.path.dirname(target_files)
    name = os.path.basename(target_files)
    name_without_ext = os.path.splitext(name)
    names = ''
    for file_path in target_files:
        name = os.path.basename(file_path)
        names += name + ' '
    os.system('cd %s && pylupdate5 -noobsolete %s -ts translations/%s.ts' % (folder, names, name_without_ext))



from PyQt5.QtWidgets import QScrollArea
from PyQt5.QtCore import Qt


class PMScrollArea(QScrollArea):
    def __init__(self):
        super().__init__()
        self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOn)
        self.setWidgetResizable(True)

    def setup_ui(self):
        if hasattr(self.widget(), 'setup_ui'):
            self.widget().setup_ui()



import os
from PyQt5.QtCore import Qt, QModelIndex, pyqtSignal, QLocale
from PyQt5.QtGui import QCursor
from PyQt5.QtWidgets import QTreeView, QFileSystemModel, QMenu, QApplication, QMessageBox, QInputDialog, \
    QLineEdit


class RewriteQFileSystemModel(QFileSystemModel):
    def __init__(self, parent=None):
        super().__init__(parent)

    def headerData(self, p_int, Qt_Orientation, role=None):
        if (p_int == 0) and (role == Qt.DisplayRole):
            return self.tr('Name')
        elif (p_int == 1) and (role == Qt.DisplayRole):
            return self.tr('Size')
        elif (p_int == 2) and (role == Qt.DisplayRole):
            return self.tr('Type')
        elif (p_int == 3) and (role == Qt.DisplayRole):
            return self.tr('Last Modified')
        else:
            return super().headerData(p_int, Qt_Orientation, role)

    def columnCount(self, parent: QModelIndex = ...) -> int:
        return 1


class PMGFilesTreeview(QTreeView):
    '''
    文件树
    '''
    new_file_signal = pyqtSignal(str)
    new_folder_signal = pyqtSignal(str)
    delete_file_signal = pyqtSignal(str)
    rename_file_signal = pyqtSignal(str, str)

    def __init__(self, initial_dir: str = '', parent=None):
        super().__init__(parent)
        self.initial_dir = initial_dir
        self.setup_ui()
        self.bind_events()

    def setup_ui(self):
        '''
        界面初始化
        :return:
        '''
        from pmgwidgets.platform.translation import add_translation_file
        path = os.path.join(os.path.dirname(__file__), 'translations', 'qt_{0}.qm'.format(QLocale.system().name()))
        self.translator = add_translation_file(path)
        self.setTabKeyNavigation(True)
        self.setDragEnabled(True)
        self.setDragDropOverwriteMode(True)
        self.setAlternatingRowColors(False)
        self.setUniformRowHeights(True)
        self.setSortingEnabled(True)
        self.setAnimated(True)
        self.setAllColumnsShowFocus(False)
        self.setWordWrap(False)
        self.setHeaderHidden(False)
        self.setObjectName("treeView_files")
        self.header().setSortIndicatorShown(True)

        self.model = RewriteQFileSystemModel()
        self.model.setRootPath(self.initial_dir)

        self.setModel(self.model)
        self.setRootIndex(self.model.index(self.initial_dir))
        self.setAnimated(False)
        self.setSortingEnabled(True)  # 启用排序
        self.header().setSortIndicatorShown(True)  # 启用标题排序
        self.setContextMenuPolicy(Qt.CustomContextMenu)
        self.customContextMenuRequested.connect(self.show_context_menu)

        self.init_context_menu()

    def bind_events(self):
        '''
        回调、事件与信号初始化
        :return:
        '''
        self.openAction.triggered.connect(self.on_open)
        self.importAction.triggered.connect(self.on_import)
        self.renameAction.triggered.connect(self.on_rename)
        self.deleteAction.triggered.connect(self.on_delete)

        self.new_file_action.triggered.connect(self.on_new_file)
        self.new_folder_action.triggered.connect(self.on_new_folder)

        self.customContextMenuRequested.connect(self.show_context_menu)

    def init_context_menu(self):
        '''
        初始化右键菜单
        :return:
        '''
        self.contextMenu = QMenu(self)
        self.openAction = self.contextMenu.addAction(self.tr('Open'))

        self.importAction = self.contextMenu.addAction(self.tr('Import'))
        self.importAction.setEnabled(False)

        self.new_file_or_folder_menu = QMenu(self.tr('New..'))
        self.contextMenu.addMenu(self.new_file_or_folder_menu)
        self.new_file_action = self.new_file_or_folder_menu.addAction(self.tr('File'))
        self.new_folder_action = self.new_file_or_folder_menu.addAction(self.tr('Folder'))
        self.new_file_or_folder_menu.addSeparator()

        self.renameAction = self.contextMenu.addAction(self.tr('Rename'))
        self.deleteAction = self.contextMenu.addAction(self.tr('Delete'))

    def show_context_menu(self):
        '''
        显示上下文右键菜单
        :return:
        '''
        self.contextMenu.popup(QCursor.pos())
        self.contextMenu.show()

    def get_current_file_path(self):
        '''
        获取当前文件的路径
        :return:
        '''
        index = self.currentIndex()
        file_info = self.model.fileInfo(index)
        return file_info.absoluteFilePath()

    def on_new_folder(self):
        '''
        新建文件夹时出发的回调
        :return:
        '''
        path = self.get_current_file_path()
        name, stat = QInputDialog.getText(self, self.tr('Please Input file name'), '', QLineEdit.Normal, '')
        if name.find('.') != -1:
            QMessageBox.critical(self, self.tr('Error'),
                                 self.tr('Folder name %s is illeagal!' % name))
            return
        if stat:
            if os.path.isdir(path):
                new_folder_path = os.path.join(path, name)
            else:
                new_folder_path = os.path.join(os.path.dirname(path), name)

            if os.path.exists(new_folder_path):
                QMessageBox.critical(self, self.tr('Error'),
                                     self.tr('Folder %s already exists!' % name))
                return
            else:
                os.mkdir(new_folder_path)
                self.new_folder_signal.emit(new_folder_path)

    def on_new_file(self):
        '''
        新建文件时触发的回调
        :return:
        '''
        path = self.get_current_file_path()
        name, stat = QInputDialog.getText(self, self.tr('Please Input file name'), '', QLineEdit.Normal, '')
        if stat:
            if os.path.isdir(path):
                new_file_path = os.path.join(path, name)
            else:
                new_file_path = os.path.join(os.path.dirname(path), name)

            if os.path.exists(new_file_path):
                QMessageBox.critical(self, self.tr('Error'),
                                     self.tr('File %s already exists!' % name))
                return
            with open(new_file_path, 'wb') as f:
                f.close()
                self.new_file_signal.emit(new_file_path)

    def on_open(self):
        '''
        点击‘open’时候触发的回调，相当于双击。
        :return:
        '''
        pass

    def on_import(self):
        '''

        :return:
        '''
        pass

    def on_rename(self):
        '''
        点击’重命名‘时候的回调。
        :return:
        '''
        from pmgwidgets.platform import rename_file
        path = self.get_current_file_path()
        basename = os.path.basename(path)
        dir_name = os.path.dirname(path)
        name, stat = QInputDialog.getText(self, self.tr('Please Input file name'), '', QLineEdit.Normal, basename)
        if stat:
            new_absolute_path = os.path.join(dir_name, name)
            rename_result = rename_file(path, new_absolute_path)
            if not rename_result:
                QMessageBox.critical(self, self.tr('Error'),
                                     self.tr('Unable to Rename this file.'))
            else:
                self.rename_file_signal.emit(path, new_absolute_path)

    def on_delete(self):
        '''
        点击’删除‘时的回调
        :return:
        '''
        from pmgwidgets.platform import move_to_trash
        path = self.get_current_file_path()

        moved_successful = move_to_trash(path)
        if not moved_successful:
            QMessageBox.critical(self, self.tr('Error'),
                                 self.tr('Unable to Move this file to recycle bin.'))
        else:
            self.delete_file_signal.emit(path)


class Stack(object):

    def __init__(self):
        # 创建空列表实现栈
        self.__list = []

    def is_empty(self):
        # 判断是否为空
        return self.__list == []

    def push(self, item):
        # 压栈，添加元素
        self.__list.append(item)

    def pop(self):
        # 弹栈，弹出最后压入栈的元素
        if self.is_empty():
            return
        else:
            return self.__list.pop()

    def top(self):
        # 取最后压入栈的元素
        if self.is_empty():
            return
        else:
            return self.__list[-1]

    def __len__(self):
        return len(self.__list)

    def __str__(self):
        return str(self.__list)


if __name__ == '__main__':
    import sys

    app = QApplication(sys.argv)

    tree = PMGFilesTreeview('c:/users/', None)
    tree.show()
    sys.exit(app.exec_())



from PyQt5.QtCore import pyqtSignal
from PyQt5.QtWidgets import QWidget


class PMDockObject(object):
    '''
    修改：
    原先，主窗口中的各个可停靠窗口，在点击右上角关闭按钮的时候会隐藏，可以在视图菜单中打开。
    但是当控件中有on_closed_action属性，且值为‘delete’的时候，控件就会被回收。
    为了实现控件的管理，控件需要继承PMDockObject，并且需要用多继承的方式。

    from pyminer2.ui.generalwidgets import PMDockObject
    这个PMDockObject中定义了一些方法，作为补充。

    class PMDockObject(object):
        on_closed_action = 'hide'  # 或者'delete'。

        def raise_widget_to_visible(self, widget: 'QWidget'):
            pass

        def on_dock_widget_deleted(self):
            pass

    '''
    on_closed_action = 'hide'  # 或者'delete'。

    def raise_widget_to_visible(self, widget: 'QWidget'):
        pass

    def on_dock_widget_deleted(self):
        pass



import queue
import subprocess
import sys
import threading
import time
import chardet
from typing import List


class PMGProcess():
    def __init__(self, args: List[str]):
        self.terminate = False
        self.q = queue.Queue()
        self.on_command_received = lambda cmd: print(cmd)
        self.on_error_received = lambda error: print(error)
        self.args = args
        self.process = subprocess.Popen(self.args,
                                        stdin=subprocess.PIPE,
                                        shell=False, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        self.to = threading.Thread(
            target=self.enqueue_stream, args=(
                self.process.stdout, self.q, 1))
        self.te = threading.Thread(
            target=self.enqueue_stream, args=(
                self.process.stderr, self.q, 2))
        self.tp = threading.Thread(target=self.consoleLoop)
        self.to.setDaemon(True)
        self.te.setDaemon(True)
        self.tp.setDaemon(True)
        self.te.start()
        self.to.start()
        self.tp.start()

    def enqueue_stream(self, stream, queue, type):  # 将stderr或者stdout写入到队列q中。
        for line in iter(stream.readline, b''):
            if self.terminate:break
            encoding = chardet.detect(line)['encoding']
            queue.put(str(type) + line.decode(encoding))
        stream.close()

    def consoleLoop(self):  # 封装后的内容。
        return
        idleLoops = 0
        while True:
            if not self.q.empty():
                line = self.q.get()
                if line[0] == '1':
                    self.on_command_received(line[1:])
                else:
                    self.on_error_received(line[1:])
                sys.stdout.flush()
            else:
                time.sleep(0.01)
                if idleLoops >= 5:
                    idleLoops = 0
                    # print('write!!')
                    self.process.stdin.write(
                        'messsage\n'.encode('ascii'))  # 模拟输入
                    self.process.stdin.flush()
                    continue
                idleLoops += 1


if __name__ == '__main__':
    pmp = PMGProcess(['python','-u', '-c','print(hello world!)'])
    while (1):
        time.sleep(2)
        pass



'''
pmpushbuttons是一个按钮组，专门负责向工具栏中插入按钮。
其中PMPushButtonPane是按钮的载体，可以插入竖向排布的两个到三个按钮。
作者：Zhanyi Hou
'''
from typing import List

from PyQt5.QtGui import QIcon
from PyQt5.QtWidgets import QWidget, QVBoxLayout, QPushButton, QMenu

from pmgwidgets.sourcemgr import create_icon


class PMPushButtonPane(QWidget):
    def __init__(self):
        super().__init__()
        self.layout = QVBoxLayout()
        self.layout.setContentsMargins(0, 0, 0, 0)
        self.layout.setSpacing(0)
        self.setLayout(self.layout)

    def add_height_occu_buttons(self):
        button_num = 3
        text = ['','','']
        if text is None:
            text = [''] * button_num


        height = 25
        font_size = 12
        btn_list = []
        for i in range(button_num):
            b = self.add_button(text=text[i], icon=create_icon(None), menu=None, height=height,
                                font_size=font_size)
            b.setMaximumWidth(1)
            btn_list.append(b)
        return btn_list

    def add_buttons(self, button_num: int = 2, text: list = None, icon_path: list = None,
                    menu: list = None) -> List[QPushButton]:
        if text is None:
            text = [''] * button_num
        if icon_path is None:
            icon_path = [None] * button_num
        if menu is None:
            menu = [None] * button_num
        if len(text) != button_num or len(
                icon_path) != button_num or len(menu) != button_num:
            raise Exception('text,icon和menu参数都必须为长为2的可迭代对象。')
        if button_num == 2:
            height = 35
            font_size = 12
        else:
            height = 25
            font_size = 12
        btn_list = []
        for i in range(button_num):
            b = self.add_button(text=text[i], icon=create_icon(icon_path[i]), menu=menu[i], height=height,
                                font_size=font_size)
            btn_list.append(b)
        return btn_list

    def add_button(self, text: str = '', icon: QIcon = None, menu: QMenu = None, height: int = 30,
                   font_size: int = 12) -> QPushButton:
        b = QPushButton()
        b.setText(text)
        if icon is not None:
            b.setIcon(icon)
        if menu is not None:
            b.setMenu(menu)
        b.setStyleSheet(
            'QPushButton{border:0px;font-size:%dpx;padding:2px 2px;width:80px;height:%dpx;text-align:left;}' % (
                font_size, height))
        self.layout.addWidget(b)
        return b




def move_to_trash(path: str) -> bool:
    '''
    将文件移动到回收站。成功返回True,失败返回False
    :param path:绝对路径。
    :return:
    '''
    import platform
    import send2trash
    if platform.system() == "Windows":
        path = path.replace('/', '\\')
    try:
        send2trash.send2trash(path)
        return True
    except:
        import traceback
        traceback.print_exc()
        return False

def rename_file(prev_absolute_path:str, new_absolute_path:str)->bool:
    '''
    重命名文件或者文件夹
    :param prev_absolute_path:之前的绝对路径名称
    :param new_absolute_path: 之后的绝对路径名称
    :return:
    '''
    import os
    try:
        os.rename(prev_absolute_path, new_absolute_path)
        return True
    except:
        import traceback
        traceback.print_exc()
        return False
if __name__ == '__main__':
    move_to_trash('C:/Users/12957/Desktop/1.jpg')



from PyQt5.QtWidgets import QGridLayout, QWidget, QSizePolicy


class PMFlowLayoutWithGrid(QGridLayout):
    '''
    流式布局，继承自QGridLayout，以Grid的方式添加widget。
    主要作用是在保证兼容代码的基础上，做到流式布局。
    '''

    def __init__(self, parent=None, column_width=100):
        super().__init__(parent)

        self.column_width = column_width
        self.widgets_list = []

    def addWidget(self, w: QWidget, row: int, column: int,
                  rowSpan: int, columnSpan: int) -> None:
        '''
        添加控件的方法。多了一个列表将所有的控件存储起来。当然，不允许重复添加。
        :param w:
        :param row:
        :param column:
        :param rowSpan:
        :param columnSpan:
        :return:
        '''
        if not w in self.widgets_list:
            self.widgets_list.append(w)
        super().addWidget(w, row, column, rowSpan, columnSpan)

    def on_resize(self):
        '''
        在界面放大缩小的时候，会将按钮重新排布。按照表格上从上到下、从左到右的顺序，就像下面这样：
        注意这个方法无法自动调用，只能依靠它的父控件。
        1 2 3 4
        5 6 7 8
        9
        :return:
        '''
        geometry = self.geometry()
        cols = int(geometry.width() / self.column_width)

        if cols == self.columnCount():
            return
        row = 0
        col = 0
        for i in range(len(self.widgets_list)):
            w = self.widgets_list[i]
            self.removeWidget(w)
            self.addWidget(w, row, col, 1, 1)
            col += 1
            if col == cols:
                row += 1
                col = 0



class PMFlowLayout(QGridLayout):
    def __init__(self, parent=None, initial_columns=3, column_width=100):
        super().__init__(parent)

        self.column_width = column_width
        self.widgets_list = []
        self.occupy_widget = QWidget()
        self.occupy_widget.setSizePolicy(QSizePolicy.Expanding,QSizePolicy.Minimum)
    def add_widget(self, w: QWidget) -> None:
        '''
        添加控件的方法。多了一个列表将所有的控件存储起来。当然，不允许重复添加。
        :param w:
        :param row:
        :param column:
        :param rowSpan:
        :param columnSpan:
        :return:
        '''

        if not w in self.widgets_list:
            self.widgets_list.append(w)
        items = len(self.widgets_list)
        geometry = self.geometry()
        cols = int(geometry.width() / self.column_width)
        if cols == 0:
            cols = 3
        current_row = int(items / cols)
        current_col = int(items % cols)
        super().addWidget(w, current_row, current_col, 1, 1)

    def on_resize(self):
        '''
        在界面放大缩小的时候，会将按钮重新排布。按照表格上从上到下、从左到右的顺序，就像下面这样：
        注意这个方法无法自动调用，只能依靠它的父控件。
        1 2 3 4
        5 6 7 8
        9
        :return:
        '''
        geometry = self.geometry()
        cols = int(geometry.width() / self.column_width)

        if cols == self.columnCount():
            return
        row = 0
        col = 0
        for i in range(len(self.widgets_list)):
            w = self.widgets_list[i]
            self.removeWidget(w)
            self.addWidget(w, row, col, 1, 1)
            col += 1
            if col == cols:
                row += 1
                col = 0
        self.removeWidget(self.occupy_widget)
        self.addWidget(self.occupy_widget,row,col,1,1)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2020/1/6 15:41
# @Author  : hzy
# @Site    :
# @File    : setup.py
# @Software: PyCharm
# @Description:

import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name = "pmgwidgets",
    version = "0.1.11",
    author = "Hou Zhanyi",
    description = "General widgets for pyminer",
    long_description = long_description,
    long_description_content_type="text/markdown",
    url = "",
    packages = setuptools.find_packages(),
    classifiers=[],
    resources=['pmgwidgets/treeviews/translations']


)



hint = 'However if your program needn\'t this package,this warning is neglectable.'
try:
    from .matplotlib.qt5agg import PMMatplotlibQt5Widget
except:
    import warnings

    warnings.warn('matplotlib is not installed. ' + hint)
    pass
try:
    from .browser.browser import QWebEngineView
except:
    import warnings

    warnings.warn('QWebengine is not installed. ' + hint)
    pass



from PyQt5.QtGui import QCloseEvent, QResizeEvent
from PyQt5.QtWidgets import QDockWidget, QMainWindow


class PMGDockWidget(QDockWidget):
    def __init__(self, name, text='', parent: QMainWindow = None):
        super().__init__(text, parent)
        self.parent = parent
        self.name = name

    def raise_into_view(self):
        '''
        将控件提升到能直接看到的位置。特别适用于两个选项卡叠在一起的情况。
        :return:
        '''
        self.setVisible(True)
        self.setFocus()
        self.raise_()






# 通用表格控件
# 作者：侯展意
# 带有加载数据集等等的功能，相对来讲比较方便。
# 以PMG作为名字开头的控件，不依赖于主界面，只是会和主界面之间传递信号。

import sys

from PyQt5.QtCore import pyqtSignal
from PyQt5.QtWidgets import QApplication, QTabWidget, QTableWidget, QTableWidgetItem, \
    QAbstractItemView

from typing import Sized,Iterable


class PMGTableWidget(QTableWidget):
    data_name: str = ''
    data_shown = pyqtSignal(str)

    def __init__(self, parent=None):
        super().__init__(parent)
        self.setRowCount(1)
        self.setColumnCount(1)
        # self.horizontalHeader().setDefaultSectionSize(30)
        self.verticalHeader().setDefaultSectionSize(30)
        self.verticalHeader().setMinimumWidth(30)
        self.horizontalHeader().setMinimumWidth(30)

    def closeEvent(self, a0: 'QCloseEvent') -> None:
        super().closeEvent(a0)

    @staticmethod
    def check_data_can_be_displayed_by_table(data: 'Sized') -> bool:
        try:
            if not hasattr(data, '__len__'):
                return True

            rows = len(data)
            max_cols = 0
            for i, row_contents in enumerate(data):
                if hasattr(row_contents, '__iter__'):
                    col_span = len(row_contents)
                    a = row_contents[i]  # 尝试index第0项。
                else:
                    col_span = 1
                if col_span > max_cols:
                    max_cols = col_span

            columns = max_cols
            for row, row_content in enumerate(data):
                if hasattr(row_content, '__iter__'):
                    for col, content in enumerate(row_content):
                        a = data[row][col]
                else:
                    a = data[row]
                return True

        except:
            return False

    def set_data_2d(self, data:'Iterable' , rows: int = None,
                    columns: int = None):

        if not hasattr(data, '__len__'):
            item = QTableWidgetItem(str(data))
            self.setItem(0, 0, item)
            return
        if rows is None or columns is None:
            rows = len(data)
            max_cols = 0
            for row_contents in data:
                if hasattr(row_contents, '__iter__'):
                    col_span = len(row_contents)
                else:
                    col_span = 1
                if col_span > max_cols:
                    max_cols = col_span
            columns = max_cols
        self.setColumnCount(columns)
        self.setRowCount(rows)
        self.setSelectionBehavior(QAbstractItemView.SelectRows)
        for row, row_content in enumerate(data):
            if hasattr(row_content, '__iter__'):
                for col, content in enumerate(row_content):
                    item = QTableWidgetItem(str(data[row][col]))
                    self.setItem(row, col, item)
            else:
                item = QTableWidgetItem(str(data[row]))
                self.setItem(row, 0, item)


class PMGTableTabWidget(QTabWidget):
    def __init__(self, parent=None):
        super().__init__(parent)


if __name__ == '__main__':
    import numpy as np

    app = QApplication(sys.argv)
    l = ['hhhhhhhhhhhhhh',
         ['a', 'v'],
         [1, 2, 3, 4],
         [3, 4, 5, 66, 7],
         [123, '333', 'ffffffff']
         ]
    demo = PMGTableWidget()

    demo.set_data_2d( l)

    demo.show()
    sys.exit(app.exec_())



from PyQt5.QtWidgets import QWidget, QVBoxLayout, QLabel, QScrollArea


from PyQt5.QtCore import Qt


class PMScrollableLabel(QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        layout = QVBoxLayout()
        self.scroll_area = QScrollArea()
        self.scroll_area.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOn)
        self.label = QLabel()
        layout_label = QVBoxLayout()
        layout_label.addWidget(self.label)
        self.scroll_area.setLayout(layout_label)

        layout.addWidget(self.scroll_area)

        self.setLayout(layout)

        self.setText = self.label.setText
        self.setWordWrap = self.label.setWordWrap


if __name__ == '__main__':
    from PyQt5.QtWidgets import QApplication
    import sys
    app = QApplication(sys.argv)
    w = PMScrollableLabel()
    w.setWordWrap(True)
    w.setText('aaaaaaaaa ' * 10000)
    w.show()
    sys.exit(app.exec_())



#!/usr/bin/env python
# -*- coding:utf-8 -*-

from PyQt5.QtGui import QColor
from PyQt5.QtWidgets import QToolButton, QGraphicsDropShadowEffect


class PMToolButton(QToolButton):
    shadow = QGraphicsDropShadowEffect()  # 实例阴影

    def __init__(self, parent=None):
        super().__init__(parent)
        self.shadow.setColor(QColor(63, 72, 204))  # 设置阴影颜色
        self.shadow.setOffset(0, 0)  # 设置阴影方向

        self.clicked.connect(self.set_btn_clicked_effect)

    def set_btn_clicked_effect(self):
        # 设置模糊度并为按钮添加阴影
        self.shadow.setBlurRadius(20)
        self.setGraphicsEffect(self.shadow)

    def unset_btn_clicked_effect(self):
        # 设置模糊度为0 间接取消阴影
        self.shadow.setBlurRadius(0)
